/***************************************************************************
                                    PDF.h
                             ------------------
    begin                : 07.12.2005
    copyright            : (C) 2005 by Frank Moosmann and INRIA
    email                : moosmann@mrt.uka.de
 ***************************************************************************/

#ifndef PDF_H
#define PDF_H

#define MAX_PDF_SIZE 500

#include <string>

class PDF {

public:
  
  enum IncMethod {imNone, imCube, imSquare, imCircle};
  static std::string asText(IncMethod method);
  static IncMethod asIncMethod(std::string);
  struct PdfParams {
    unsigned int initValue;
    unsigned int maxValue;
    IncMethod incMethod;
  };

  PDF(unsigned int imgWidth, unsigned int imgHeight, unsigned int minScale, unsigned int maxScale, PdfParams pdfParams, std::string groundtruthfile = "");
  ~PDF();
  
  void randomSample(unsigned int* xPos, unsigned int* yPos, unsigned int* scale);
  void increaseValues(unsigned int scale, unsigned int xPos, unsigned int yPos, int explorationAmount, int objectAmount, double param1 = 0.0, double param2 = 0.0);
  void printIntoFile(std::string filename);  // for debug purposes
  void printAsImage(std::string filename);  // for debug purposes
  double relObjSize; // if groundtruth-information was provided in constructor then create statistical information out of groundtruth

private:
  void printIntoFile(std::string filename, unsigned int ***distribution);  // for debug purposes
  void printAsImage(std::string filename, unsigned int ***distribution);  // for debug purposes
  void increaseCube(unsigned int scale, unsigned int xPos, unsigned int yPos, int explorationAmount, int objectAmount, unsigned int radius);
  void increaseSquare(unsigned int scale, unsigned int xPos, unsigned int yPos, int explorationAmount, int objectAmount, unsigned int radius, unsigned int scaleradius);
  void increaseCircle(unsigned int scale, unsigned int xPos, unsigned int yPos, int explorationAmount, int objectAmount, unsigned int radius);
  void increaseValue(unsigned int s, unsigned int x, unsigned int y, int explorationAmount, int objectAmount);
  unsigned int ***explorationDistribution;
  unsigned int ***objectDistribution;
  unsigned int ***samplingDistribution; // =explorationDistribution*objectDistribution
  unsigned int **sumTable; // contains the sum of one row of samplingDistribution
  unsigned int *sumsumTable; // contains the sum of one plane of samplingDistribution
  unsigned int sumsumsum; // contains the total sum of samplingDistribution
  unsigned int xMax; //xMin=0
  unsigned int yMax; //yMin=0
  unsigned int sMin;
  unsigned int sMax;
  unsigned int sRange;
  unsigned int resolution; // for large images don't use one value for each pixel, but one value for "resolution" pixels
  PdfParams params;
};
#endif //PDF_H
