//*************************************************************************
//                        FRank's IMage CLAssifier
//                        
//                     Utility functions for main file
//                           -------------------
//  begin                : 04.11.2005
//  copyright            : (C) 2005 by Frank Moosmann and INRIA
//  email                : moosmann@mrt.uka.de
//**************************************************************************
enum ClassificationMethod {CM_EXTRA_TREE_SINGLE_ATT, CM_EXTRA_TREE_SOFT_THRESHOLD, CM_EXTRA_TREE_COMPARE_ATT, CM_SVM};
string asText(ClassificationMethod method) {
  switch (method) {
    case CM_EXTRA_TREE_SINGLE_ATT: return "EXTRATREE";
    case CM_EXTRA_TREE_SOFT_THRESHOLD: return "EXTRATREE_SOFT_THRESHOLD";
    case CM_EXTRA_TREE_COMPARE_ATT: return "EXTRATREE_2ATT";
    case CM_SVM: return "SVM";
    default: return "";
  }
}
ClassificationMethod asClassificationMethod(string method) {
  if (method == "EXTRATREE") return CM_EXTRA_TREE_SINGLE_ATT;
  if (method == "EXTRATREE_SOFT_THRESHOLD") return CM_EXTRA_TREE_SOFT_THRESHOLD;
  if (method == "EXTRATREE_2ATT") return CM_EXTRA_TREE_COMPARE_ATT;
  if (method == "SVM") return CM_SVM;
  return CM_EXTRA_TREE_SINGLE_ATT;
}


enum ClassifierStage2 {CL2_NONE, CL2_BINARY, CL2_NODE_COUNT, CL2_PURE_BINARY, CL2_PURE_NODE_COUNT, CL2_ENTROPY_PURE_BINARY, CL2_PURE_NORM_NODE_COUNT};
std::string asText(ClassifierStage2 method) {
  switch (method) {
    case CL2_NONE: return "NONE";
    case CL2_BINARY: return "BINARY";
    case CL2_PURE_BINARY: return "PUREBINARY";
    case CL2_NODE_COUNT: return "COUNT";
    case CL2_PURE_NODE_COUNT: return "PURECOUNT";
    case CL2_ENTROPY_PURE_BINARY: return "PUREBINARYENTROPY";
    case CL2_PURE_NORM_NODE_COUNT: return "PURENORMCOUNT";
    default: return "";
  }
}
ClassifierStage2 asClassifierStage2(string method) {
  if (method.compare("NONE")==0)
    return CL2_NONE;
  if (method.compare("BINARY")==0)
    return CL2_BINARY;
  if (method.compare("PUREBINARY")==0)
    return CL2_PURE_BINARY;
  if (method.compare("COUNT")==0)
    return CL2_NODE_COUNT;
  if (method.compare("PURECOUNT")==0)
    return CL2_PURE_NODE_COUNT;
  if (method.compare("PUREBINARYENTROPY")==0)
    return CL2_ENTROPY_PURE_BINARY;
  if (method.compare("PURENORMCOUNT")==0)
    return CL2_PURE_NORM_NODE_COUNT;
  return CL2_NONE;
}

// to make a list with all training files including their details
struct TrainingFile { 
  string fileName;
  string groundtruthImgFile;
  string groundtruthTxtFile;
  ClassLabel classLabel;
};

// to make a list with all classificationfiles including their desired class label
struct ClassificationFile { 
  string fileName;
  ClassLabel classLabel;
};

struct MainParams {
  string currentDirectory;
  ClassificationMethod classificationMethod;
  unsigned int nbOfTrainingWindowsTotal;  // number of feature vectors to extract for training in total
  unsigned int nbOfTrainingWindowsPerImg; // number of feature vectors per image in Training stage (calculated from above)
  bool useSameNbTrWinPerClass;
  unsigned int nbOfTestingWindowsPerImg;  // number of feature vectors to extract from each image in testing stage
  int maxSubwindowSize;                   // maximum size of sampled subwindow in % of image size
  double minObjBgrRatio;                  // minimum percentage of object in training window (using groundtruth-masks)
  string masksPath;
  string siftCfgFile;
  string groundtruthImgExtension;
  string groundtruthTxtExtension;
  FeatureExtractor::ExtractionMethod extMethod;
  double pdfIncParam1;                    // radius for cube
  double pdfIncParam2;
  int pdfIncAmount;
  int pdfDecAmount;
  bool usePDFRadiusAdaption;
  ClassifierStage2 use2ndStageClassifier;
  list<string> ImgFileExtensions;
  int nbDebugImages;                      // number of output debug images per class
  string debugImageDirectory;
  int nbPDFImages;                        // number of output debug images per class
  string PDFImageDirectory;
  string outputFile;
  vector<TrainingFile> trainingFiles;
  vector<ClassificationFile> classificationFiles;
  ClassLabelHistogram trainingFileLabels;
  ClassLabelHistogram classificationFileLabels;
  string loadClassifierFile;
  string saveClassifierFile;
  string treeLeafesOutputFile;
  bool saveFeatureVectors;
  string featureVecOutputDir;
  bool readFeatureVectors;
  string featureVecInputDir;
  bool doTreeWeightUpdate;
};

// to calculate the ROC-curve and EER
struct answer {
  int label; // +/-1
  double value;
  inline bool operator<(const answer& a) const { return value < a.value; }
};

// sets the classLabel-property of all FeatureVectors in the List and adds them to the target list
void attachClassLabel (list<FeatureVector*>* sourceFeatureList, list<FeatureVector*>* targetFeatureList, ClassLabel label) {
  for(list<FeatureVector*>::iterator p=sourceFeatureList->begin(); (p!=sourceFeatureList->end()); ++p)  {
    FeatureVector* cFeatVec = *p;
    cFeatVec->classLabel = label;
    targetFeatureList->push_back(cFeatVec);
  }
}

// needed to sort a list of TerminalNodes
extern "C" int compareTNs(TerminalNode*, TerminalNode*); extern "C" {
  int compareTNs(TerminalNode *x, TerminalNode *y)
  {
    if (x->getID() < y->getID()) return 1;
    return 0;
  }
}

// creates a feature vector for the 2nd stage classifier
FeatureVector* createFeatureVector(list<TerminalNode*> *tnList, ClassifierStage2 cls2, unsigned int maxID, ClassLabel classLabel, ClassLabel classifiedLabel) {
  bool binary = (cls2==CL2_BINARY) || (cls2==CL2_PURE_BINARY);
  bool pure = (cls2==CL2_PURE_BINARY) || (cls2==CL2_PURE_NODE_COUNT) || (cls2==CL2_ENTROPY_PURE_BINARY) || (cls2==CL2_PURE_NORM_NODE_COUNT);
  
  FeatureVector* rslt = new FeatureVector();
  rslt->classLabel = classLabel;
  if (!pure) {
    rslt->addElement(classifiedLabel);
  }
  
  tnList->sort(compareTNs);
  double nodeCount = (double)tnList->size();
  list<TerminalNode*>::iterator tn = tnList->begin();
  for (unsigned int i=1; i<=maxID; i++)  {
    double val = 0.0;
    if (!((tn == tnList->end()) || ((*tn)->getID() != i))) {
      do {
        tn++;
        if (binary)
          val = 1.0;
        else
          val++;
      }
      while ((tn != tnList->end()) && ((*tn)->getID() == i));
    }
    if (!binary)
      val /= nodeCount;
    rslt->addElement(val);
  }
  return rslt;
}

// Reads a feature vectors from a file and returns the number of features that were read
int readFileFeatureVectors (string fileName, string directory, list<FeatureVector*> &featureList, int nb=0) {
  string dataname = directory + extractFileName(fileName) + ".desc";
  string infoname = directory + extractFileName(fileName) + ".info";
  ifstream dataFile(dataname.c_str());
  ifstream infoFile(infoname.c_str());
  int result = 0;
  if ((dataFile) && (infoFile)) {
    result = FeatureVector::readFromStream(dataFile, infoFile, featureList, nb);
  }
  dataFile.close();
  infoFile.close();
  return result;
}

void saveFileFeatureVectors(string fileName, list<FeatureVector*> &featureList, string directory) {
  string dataname = directory + extractFileName(fileName) + ".desc";
  string infoname = directory + extractFileName(fileName) + ".info";
  ofstream datafile(dataname.c_str(), ios::out);
  ofstream infofile(infoname.c_str(), ios::out);
  FeatureVector::writeIntoStream(featureList, datafile, infofile);
  datafile.close();
  infofile.close();
}

// frees memory of all feature vectors contained in this list. 
// Other pointers to the feature vectors must be deleted manually!!!
void deleteFeatureVectors (list<FeatureVector*> *featureList) {
  for (list<FeatureVector*>::iterator f = featureList->begin(); f!=featureList->end(); f++)
    delete *f;
  featureList->clear();
}

// to create debug images, associate one color to each category
string getClassColor(ClassLabel c) {
  switch (c) {
    case 0: return "black";
    case 1: return "yellow";
    case 2: return "red";
    case 3: return "blue";
    case 4: return "green";
    default: return "pink";
  }
}

// method to create a debug images containing the windows, that were extracted from it
void createDebugImage(string sourceImg, string debugDirectory, list<FeatureVector*> *featureList, ClassLabelHistogram* totalVotes, ClassLabelHistogram* windowVotes, list<ClassLabel>* classList, FeatureExtractor::ExtractionMethod extMethod) {
  Magick::Image debugImg;
  std::list<Magick::Drawable> drawList;
  debugImg.read(sourceImg.c_str());
  if (extMethod == FeatureExtractor::Gradient)
    debugImg.edge(0.);
  drawList.push_back(DrawableFillColor("transparent")); // Fill color
  drawList.push_back(DrawableStrokeWidth(1));
  for(list<FeatureVector*>::iterator p=featureList->begin(); (p!=featureList->end()); ++p)  {
    FeatureVector* cFeatVec = *p;
    drawList.push_back(DrawableStrokeColor(getClassColor(cFeatVec->classLabel)));
    drawList.push_back(DrawableRectangle(cFeatVec->windowPosX, cFeatVec->windowPosY, cFeatVec->windowPosX+cFeatVec->windowSize, cFeatVec->windowPosY+cFeatVec->windowSize));
  }
  if ((totalVotes != 0) && (windowVotes != 0) && (classList != 0)) {
    //debugImg.fontPointsize = 12;
    unsigned int yPos = 0;
    unsigned int lineHeight = 28;
    drawList.push_back(DrawablePointSize(24));// set the point size
    drawList.push_back(DrawableStrokeColor("black"));
    drawList.push_back(DrawableFillColor(getClassColor(0)));
    drawList.push_back(DrawableText(15, (++yPos)*lineHeight, 
                         to_String(windowVotes->getCountOfLabel(0))+" ("+to_String(totalVotes->getCountOfLabel(0))+")"));
    for (list<ClassLabel>::iterator p=classList->begin(); (p!=classList->end()); ++p)  {
      //drawList.push_back(DrawableStrokeColor(getClassColor(*p)));
      drawList.push_back(DrawableFillColor(getClassColor(*p)));
      drawList.push_back(DrawableText(15, (++yPos)*lineHeight, 
                         to_String(windowVotes->getCountOfLabel(*p))+" ("+to_String(totalVotes->getCountOfLabel(*p))+")"));
    }
  }
  debugImg.draw(drawList);
  debugImg.write(string(debugDirectory+extractFileName(sourceImg)+"-window.jpg").c_str());
}   

// method to create a debug images of the PDF
void createPDFImage(string sourceImg, string debugDirectory, PDF *pdf) {
  pdf->printAsImage(string(debugDirectory+extractFileName(sourceImg)+"-pdf"));
  //pdf->printIntoFile(string(debugDirectory+extractFileName(sourceImg)+"-pdf"));
}   

// not used any more! reads textfile and creates a maskFile for FeatureExtactor
void createGroundtruthImage(string sourceFile, string groundtruthFile, string outputFilename) {
  Magick::Image master(sourceFile.c_str());
  unsigned int width = master.columns();
  unsigned int height = master.rows();
  string icString = to_String<unsigned int>(width)+"x"+to_String<unsigned int>(height);
  Magick::Image image( icString, "white" );
  std::list<Magick::Drawable> drawList;
  drawList.push_back(DrawableFillColor("black"));
  drawList.push_back(DrawableStrokeColor("black"));
  drawList.push_back(DrawableStrokeWidth(1));
  ifstream inFile (groundtruthFile.c_str());
  if (!inFile) {
    cerr << "Unable to open groundtruthfile";
    exit(1);
  }
  string s;
  while (! inFile.eof() )
  {
    getline (inFile,s);
    if (s.length() > 7) {
      vector<std::string> params;
      try {
        explode(s, " ", &params);
      } catch (Exception &error_) {
        continue;
      }
      int p0; int p1; int p2; int p3;
      from_string<int>(p0, params[0], std::dec); // presumably missing p0 entry added by Bill 19/09/07
      from_string<int>(p1, params[1], std::dec);
      from_string<int>(p2, params[2], std::dec);
      from_string<int>(p3, params[3], std::dec);
      //drawList.push_back(DrawableRectangle(p0, p1, p2+p0, p3+p1));
      drawList.push_back(DrawableRectangle(p0, p1, p2, p3)); //upperLeftX_, upperLeftY_, lowerRightX_, lowerRightY
    }
  }
  inFile.close();
  image.draw(drawList);
  image.write(outputFilename.c_str());
}   

// method extracts features and classifies the image. used for training the 2nd-stage-classifier and for testing
void classifyImage(Classifier *classifier, ExtraTreeClassifier *etclassifier, string fileName, 
                   bool createDebugImg, bool createPDFImg, MainParams mainParams, PDF::PdfParams pdfParams,
                   ClassLabelHistogram *totalVotes, ClassLabelHistogram *windowVotes, list<TerminalNode*> *tnList) {
  
  list<FeatureVector*> totalFeatureList;
  unsigned int nbClassifiedFeat = 0; // total number of features extracted
  
  // ======================== read features from file =======================
  if (mainParams.readFeatureVectors) {
    // ------------------------ read features ---------------------------
    nbClassifiedFeat = readFileFeatureVectors(fileName, mainParams.featureVecInputDir, totalFeatureList, mainParams.nbOfTestingWindowsPerImg);
    // ------------------------ classify --------------------------------
    if (etclassifier == 0)
      classifier->classify(&totalFeatureList, totalVotes, windowVotes);
    else {
      etclassifier->classify(&totalFeatureList, tnList, (mainParams.treeLeafesOutputFile != ""), totalVotes, windowVotes);
    }
  } else {
  // ======================= extract features from image ====================
    Magick::Image img;
    try {
      img.read(fileName.c_str());
    } catch( Exception &error_ ) { 
      cerr << "Caught exception while reading image: " << error_.what() << endl; 
      return;
    } // try reading the image
    unsigned int maxScale = (unsigned int)(((double)min(img.columns(),img.rows()) * mainParams.maxSubwindowSize / 100));
    PDF* pdf = new PDF(img.columns(), img.rows(), WINDOW_SIZE, maxScale, pdfParams);
    FeatureExtractor* fExtractor = new FeatureExtractor(img, fileName, pdf, mainParams.extMethod, mainParams.siftCfgFile);
    unsigned int nbAcceptedFeat = 0;   // only accept a feature if it does not vote "undecided"
    //while ((nbAcceptedFeat < mainParams.noOfTestingWindowsPerImg) && (nbClassifiedFeat < 3*mainParams.nbOfTestingWindowsPerImg))
    while (nbClassifiedFeat < mainParams.nbOfTestingWindowsPerImg)
    {
      list<FeatureVector*> featureList; featureList.clear();
      // --------------------- extract Feature ---------------------------
      fExtractor->extractFeatures(1, &featureList);
      FeatureVector* currFVec = *(featureList.begin());
      // decrement sampling probability at this point:
      pdf->increaseValues(currFVec->windowSize, currFVec->windowPosX, currFVec->windowPosY, -20, 0, 0.5*mainParams.pdfIncParam1, 0.8*mainParams.pdfIncParam1); // (scale, xPos, yPos, amount, method, radius)
      pdf->increaseValues(currFVec->windowSize, currFVec->windowPosX, currFVec->windowPosY, -30, 0, 0.2*mainParams.pdfIncParam1, 0.4*mainParams.pdfIncParam1); // (scale, xPos, yPos, amount, method, radius)
      // to create debug image save the feature in total feature list
      totalFeatureList.push_back(currFVec);
      // ------------------------ classify --------------------------------
      ClassLabel classifiedLabel;
      list<TerminalNode*> fileTnList; fileTnList.clear();
      if (etclassifier == 0)
        classifiedLabel = classifier->classify(&featureList, totalVotes, windowVotes);
      else {
        classifiedLabel = etclassifier->classify(&featureList, &fileTnList, (mainParams.treeLeafesOutputFile != ""), totalVotes, windowVotes);
				// totalVotes contains weightes sum of all votes, windowVotes contains #subwindows votes unweighted
        for(list<TerminalNode*>::iterator tn=fileTnList.begin(); tn!=fileTnList.end(); ++tn)  {
          tnList->push_back(*tn);
        }
      }
      nbClassifiedFeat++;
      // ------------------------ adjust PDF --------------------------------
      if (classifiedLabel > 0) {
        nbAcceptedFeat++;
        double weight = 0;
        unsigned int tncount = 0;
        for(list<TerminalNode*>::iterator tn=fileTnList.begin(); tn!=fileTnList.end(); ++tn)  {
          if ((*tn)->classLabel == classifiedLabel) {
            tncount++;
            //weight = max(weight, (*tn)->weight);
            weight += (*tn)->weight;
          }
        }
        if (tncount > 0)
          weight /= tncount;
        if (tncount == 0)
          weight = 1;
        if (classifiedLabel > 1) {
          // adjust PDF:
          double pdfIncRadius = mainParams.pdfIncParam1;
          if ((mainParams.usePDFRadiusAdaption) && (tncount > 0)) {
            unsigned int ratiocount = 0;
            double w2oRatio = 0;
            for(list<TerminalNode*>::iterator tn=fileTnList.begin(); tn!=fileTnList.end(); ++tn)  {
              if ((*tn)->windowToObjectRatio > 0) {
                ratiocount++;
                w2oRatio += (*tn)->windowToObjectRatio;
              }
            }
            w2oRatio /= ratiocount;
            // FeatureExtractor.cpp: windowToObjectRatio = windowScale^2 / objWidth*objHeight
            // --> objScale = windowScale / sqrt(windowToObjectRatio)
            // PDF.cpp: incRadiusPix = incRadiusRatio * min(im_width,im_height) / 100
            // ==> incRadiusRatio = (objScale / 2) * 100 / min(im_width,im_height) = windowSize * 50 / sqrt(windowToObjectRatio) / min(im_width,im_height)
            pdfIncRadius = currFVec->windowSize * 50 / sqrt(w2oRatio) / min(currFVec->imageWidth, currFVec->imageHeight);
          }
          //cout << (weight*(double)mainParams.pdfIncAmount) << "  " << flush;
          pdf->increaseValues(currFVec->windowSize, currFVec->windowPosX, currFVec->windowPosY, 0, (unsigned int)(weight*(double)mainParams.pdfIncAmount), pdfIncRadius, mainParams.pdfIncParam1); // (scale, xPos, yPos, amount, method, radius)
        } else {
          //cout << (weight*(double)mainParams.pdfDecAmount) << "  " << flush;
          pdf->increaseValues(currFVec->windowSize, currFVec->windowPosX, currFVec->windowPosY, 0, (unsigned int)(weight*(double)mainParams.pdfDecAmount), mainParams.pdfIncParam1, mainParams.pdfIncParam2); // (scale, xPos, yPos, amount, method, radius)
        }
      } //-end if (classifiedLabel > 0)
			if ((createPDFImg) && (nbClassifiedFeat%100 == 0))
				createPDFImage(fileName + to_String(nbClassifiedFeat), mainParams.debugImageDirectory, pdf);
    } //-end while ((nbAcceptedFeat < mainParams.nbOfTestingWindowsPerImg) && (nbClassifiedFeat < 3*mainParams.nbOfTestingWindowsPerImg))
    if (createDebugImg)
      createDebugImage(fileName, mainParams.debugImageDirectory, &totalFeatureList, totalVotes, windowVotes, &(classifier->classLabelList), mainParams.extMethod);
    if (createPDFImg)
      createPDFImage(fileName, mainParams.debugImageDirectory, pdf);
    delete fExtractor;
    delete pdf;
  } //-end extract features
  
  if (nbClassifiedFeat < mainParams.nbOfTestingWindowsPerImg) {
    cerr << endl << "Warning: could only extract " << nbClassifiedFeat << " features from image " << extractFileName(fileName) << flush;
  }
  
  if (mainParams.saveFeatureVectors) {
    saveFileFeatureVectors(fileName, totalFeatureList, mainParams.featureVecOutputDir);
  }
  
  deleteFeatureVectors(&totalFeatureList);
}

// method to calculate thresholds for optimal binarisation
void calcBinarisationThresholds(list<FeatureVector*> &featureList, vector<double> &binariseThresholds, list<ClassLabel> &classLabelList) {
  // split features into lists for each class
  list< list<FeatureVector*>* > splittedFeatures;
  double fcount = 0;
  int dimcount = (*(featureList.begin()))->getSize();
  for (list<ClassLabel>::iterator c=classLabelList.begin(); c!=classLabelList.end(); ++c)  {
    ClassLabel currClass = *c;
    list<FeatureVector*> *currClassFeatures = new list<FeatureVector*>;
    for(list<FeatureVector*>::iterator p=featureList.begin(); (p!=featureList.end()); ++p)  {
      fcount++;
      if ((*p)->classLabel == currClass)
        currClassFeatures->push_back(*p);
    }
    splittedFeatures.push_back(currClassFeatures);
  }
  
  // now calculate the best threshold for each dimension
  // according to H(S) = - SUM_class ( p_class * SUM_val (p_val * log2 p_val ) )
  for (int dim=0; dim<dimcount; dim++) {
    //cout << endl << endl << "DIMENSION " << dim << "   ";
    list<double> values;
    // create list of possible threshold values
    values.clear();
    for(list<FeatureVector*>::iterator p=featureList.begin(); (p!=featureList.end()); ++p) {
      values.push_back((*p)->getElement(dim));
      //cout << (*p)->getElement(dim) << "  ";
    }
    //values.sort();
    double currT = 0.0;
    double maxscore = 0.0;
    double maxT = 0.0;
    // try every possible threshold to figure out which one is best
    for(list<double>::iterator t=values.begin(); t!=values.end(); ++t)  {
      //cout << endl << "current Threshold: " << *t << "   ";
      if ((*t != currT) || (t==values.begin())) {
        currT = *t;
        //binarise and calculate new score
        double score = 0.0;
        for (list< list<FeatureVector*>* >::iterator l=splittedFeatures.begin(); l!=splittedFeatures.end(); ++l)  {
          list<FeatureVector*> *currList = *l;
          double n1 = 0;
          double n0 = 0;
          for(list<FeatureVector*>::iterator f=currList->begin(); (f!=currList->end()); ++f)  {
            if ((*f)->getElement(dim) > currT)
              n1++;
            else
              n0++;
          } //-end for feature vector in this class
          double n = n1 + n0;
          //cout << n0 << "/" << n1;
          double innerscore = 0.0;
          if ((n0 != 0) && (n1 != 0)) {
            innerscore += n0/n * log2 (n0/n);      // = p_0 log2 (p_0)
            innerscore += n1/n * log2 (n1/n);      // = p_1 log2 (p_1)
            innerscore *= n/fcount;                // = p_class
            score -= innerscore;
          }
          //cout << "-->s:" << innerscore << "  ";
        } //-end for each class
        //cout << ",    current score:" << score;
        if (score > maxscore) {
          //cout << "-> override:" << maxscore;
          maxscore = score;
          maxT = currT;
        }
      } //-end if threshold different from last one
    } //-end for every possible threshold
    //cout << endl << " pushed threshold: " << maxT;
    binariseThresholds.push_back(maxT);
  } //-end for each dimension
  
  // delete the lists for each class
  for (list< list<FeatureVector*>* >::iterator l=splittedFeatures.begin(); l!=splittedFeatures.end(); ++l)  {
    delete (*l);
  }
}

// method to binarise histograms
void binarise(list<FeatureVector*> &featureList, vector<double> &binariseThresholds) {
  for(list<FeatureVector*>::iterator p=featureList.begin(); (p!=featureList.end()); ++p)  {
    FeatureVector *cFeatVec = *p;
    int size = min(cFeatVec->getSize(), (unsigned int)binariseThresholds.size());
    for (int dim=0; dim<size; dim++) {
      if (cFeatVec->getElement(dim) > binariseThresholds[dim])
        cFeatVec->setElement(dim,1);
      else
        cFeatVec->setElement(dim,0);
    }
  }
}

// method to calculate thresholds for optimal binarisation
void calcNormalizationParameters(list<FeatureVector*> &featureList, vector<double> &means, vector<double> &deviations) {
  int dimcount = (*(featureList.begin()))->getSize();
  for (int dim=0; dim<dimcount; dim++) {
    //cout << endl << endl << "DIMENSION " << dim << "   ";
    double mean = FeatureVector::compEmpMean(featureList, dim);
    double dev = FeatureVector::compStdDeviation(featureList, dim, mean);
    if (dev == 0)
      dev = 1;
    means.push_back(mean);
    deviations.push_back(dev);
  } //-end for each dimension
}

// method to binarise histograms
void normalize(list<FeatureVector*> &featureList, vector<double> &means, vector<double> &deviations) {
  int size = min(min((*(featureList.begin()))->getSize(), (unsigned int)means.size()), (unsigned int)deviations.size());
  for(list<FeatureVector*>::iterator p=featureList.begin(); (p!=featureList.end()); ++p)  {
    FeatureVector *cFeatVec = *p;
    for (int dim=0; dim<size; dim++) {
      double val = cFeatVec->getElement(dim);
      val = (val - means[dim]) / deviations[dim];
      cFeatVec->setElement(dim,val);
    }
  }
}

// method to print test summary. used for cout and outputfile
void printSummary(ostream *out, MainParams mainParams, ExtraTreeClassifier::ExtraTreeParams extraTreeParams, PDF::PdfParams pdfParams, int diffwindowing, int difftraining, int diffwindowing2, int difftraining2, int diffclassification, list<ClassLabel> *classLabelList,
                  unsigned int** confusionMatrix, unsigned int nbCorrectlyClassified, unsigned int nbMisclassified, double EqualErrorRate, double tprate, double EqualErrorRate2, double tprate2) {
  (*out) << endl;
  (*out) << "XXXXXXXXXXXXXXXXXXXXXXXXXXXX"<< endl;
  (*out) << "X          summary         X"<< endl;
  (*out) << "XXXXXXXXXXXXXXXXXXXXXXXXXXXX"<< endl;
  double errorRate = -1.0;
  if ((nbMisclassified>0) || (nbCorrectlyClassified > 0))
    errorRate = 100.0*((double)nbMisclassified)/((double)(nbMisclassified+nbCorrectlyClassified));
  (*out) << "Configuration: " << endl;
  (*out) << "  ExtractionMethod: " << FeatureExtractor::asText(mainParams.extMethod) << endl;
  (*out) << "  maxSubwindowSize: " << mainParams.maxSubwindowSize << "%" << endl;
  (*out) << "  minObjBgrRatio: " << mainParams.minObjBgrRatio << endl;
  //(*out) << "  groundtruthImgExtension: " << groundtruthImgExtension << endl;
  //(*out) << "  groundtruthTxtExtension: " << groundtruthTxtExtension << endl;
  (*out) << "  nbOfTrainingWindowsTotal: " << mainParams.nbOfTrainingWindowsTotal << endl;
  (*out) << "  nbOfTrainingWindowsPerImg: " << mainParams.nbOfTrainingWindowsPerImg << endl;
  (*out) << "  nbOfTestingWindowsPerImg: " << mainParams.nbOfTestingWindowsPerImg << endl;
  (*out) << "  ClassificationMethod: " << asText(mainParams.classificationMethod) << endl;
  if ((mainParams.classificationMethod == CM_EXTRA_TREE_SINGLE_ATT)
  || (mainParams.classificationMethod == CM_EXTRA_TREE_COMPARE_ATT)
  || (mainParams.classificationMethod == CM_EXTRA_TREE_SOFT_THRESHOLD)) {
    (*out) << "    nbOfTrees: " << extraTreeParams.nbOfTrees << endl;
    (*out) << "    score function: min" << extraTreeParams.scoreThreshold << ", " << extraTreeParams.nbScoreTries << " tries" << endl;
    (*out) << "    pruningThreshold: " << extraTreeParams.pruningThreshold << endl;
    (*out) << "    pruningLimit: " << extraTreeParams.maxNbPrunedNodes << "%" << endl;
    (*out) << "    maxNbTerminalNodes: " << extraTreeParams.maxNbTerminalNodes << endl;
  }
  (*out) << "  2nd-Stage-Classifier: " << asText(mainParams.use2ndStageClassifier);
  if (mainParams.doTreeWeightUpdate)
    (*out) << ", tree-weight-update used";
  (*out) << endl;
  (*out) << "  PDF-Initialisation-Value: " << pdfParams.initValue << ", Maximum Value " << pdfParams.maxValue << endl;
  (*out) << "  PDF-Incrementation: " << PDF::asText(pdfParams.incMethod) << ", radius " << mainParams.pdfIncParam1 << "%, value " << mainParams.pdfIncAmount << "/" << mainParams.pdfDecAmount;
  if (mainParams.usePDFRadiusAdaption)
    (*out) << ", PDF-Radius-Adaption used";
  (*out) << endl;
  (*out) << "Results: " << endl;
  (*out) << "  " << mainParams.trainingFiles.size() << " training files processed in " << diffwindowing+difftraining+diffwindowing2+difftraining2 << "s (feature extraction: " << diffwindowing << "s+" << diffwindowing2 << "s, training: " << difftraining << "s+" << difftraining2 << "s)" << endl;
  (*out) << "  " << mainParams.classificationFiles.size() << " testing files processed in " << diffclassification << "s" << endl;
  if ((mainParams.classificationMethod == CM_EXTRA_TREE_SINGLE_ATT) || (mainParams.classificationMethod == CM_EXTRA_TREE_COMPARE_ATT)) {
    (*out) << "  Terminal node class distribution:  ";
    (*out) << TerminalNode::classDistribution->getCountOfLabel(0);
    for (list<ClassLabel>::iterator p=classLabelList->begin(); (p!=classLabelList->end()); ++p)  {
      (*out) << "   " << TerminalNode::classDistribution->getCountOfLabel(*p);
    } (*out) << endl;
  }
  if (errorRate >= 0.0) {
    (*out) << "  Confusion Matrix:" << endl;
    (*out) << "\t";
    for (list<ClassLabel>::iterator q=classLabelList->begin(); (q!=classLabelList->end()); ++q)  {
      (*out) << "\t" << *q;
    }
    (*out) << endl;
    for (list<ClassLabel>::iterator p=classLabelList->begin(); (p!=classLabelList->end()); ++p)  {
      (*out) << "\t" << *p << ":";
      for (list<ClassLabel>::iterator q=classLabelList->begin(); (q!=classLabelList->end()); ++q)  {
        (*out) << "\t" << confusionMatrix[*p][*q];
      }
      (*out) << endl;
    }
    (*out) << "  Error rate: " << errorRate << "% (correct: " << nbCorrectlyClassified << ", wrong: " << nbMisclassified << ")" << endl;
    if (EqualErrorRate >= 0)
      (*out) << "  Equal Error rate1: " << EqualErrorRate << "% (at True-Positive-Rate " << tprate << ")" << endl;
    if (EqualErrorRate2 >= 0)
      (*out) << "  Equal Error rate2: " << EqualErrorRate2 << "% (at True-Positive-Rate " << tprate2 << ")" << endl;
    (*out) << endl << endl;
  }
}

void createROCPlot(string filename, double *tn, double *fn, double *tp, double *fp, answer *answers, int nbFiles, double *EqualErrorRate, double *tprate) {
  ofstream outFileROC;
  outFileROC.open(filename.c_str(), ios::out);
  outFileROC << (*fp/(*tn+*fp)) << " " << (*tp/(*tp+*fn)) << endl;
  sort(answers,answers+nbFiles);
  bool EERdetected = false;
  for (int j=nbFiles;j>0;j--)
  {
    int currLabel = answers[j-1].label;
    if (currLabel > 0)
    {
      *tp += currLabel; //+1
      *fn -= currLabel; //-1
    } else {
      *fp -= currLabel; //+1
      *tn += currLabel; //-1
    }
    double x = *fp/(*fp+*tn);
    double y = *tp/(*tp+*fn);
    if (y > 1-x)
      EERdetected = true;
    if (!EERdetected) {
      *EqualErrorRate = (double)(*fp+*fn)*100/nbFiles; //nbFiles=(tp+fp+tn+fn)
      *tprate = y;
    }
    if (j==0 || answers[j-1].value != answers[j].value) 
      outFileROC << x << " " << y << endl;
  }
  outFileROC.close();
}
