// method to parse the argv and set the params, returns string with error message
string evaluateParams(vector<string> &args, MainParams *mainParams, ExtraTreeClassifier::ExtraTreeParams *extraTreeParams, svm_parameter *svmParams, PDF::PdfParams *pdfParams) {
  bool paramError = false;
  int parseState = 0;
  string currFile;
  string errorMsg = "";
  int currInt1 = 0;
  int currInt2 = 0;
  int argcnt = args.size();
  for (int i=0; ((i<argcnt)&&(!paramError)); i++) {
    string actParam = args[i];
    //cout << "paramError="<<paramError<<", parsing "<<actParam<<endl;
    
    if (actParam.compare("--help")==0) {
      cout << "Usage: frimcla " 
         << endl << " [--configFile filename]"
#ifdef HAVE_SIFT
         << endl << " [--SiftConfigFile filename]"
#endif
#ifdef HAVE_SIFT
         << endl << " [--extractionMethod HSL|RGB|GREY|WaveletHSL|WaveletRGB|WaveletGREY|Gradient|SIFT|normSIFT|SIFTHSL|SIFT+meanHSL]"
#else
         << endl << " [--extractionMethod HSL|RGB|GREY|WaveletHSL|WaveletRGB|WaveletGREY|Gradient]"
#endif
         << endl << " [--maxSubwindowSize percent(int)]"
         << endl << " [--totalTrainingFeatures number(int)]"
         << endl << " [--equalizeTrainingFeaturesPerClass]"
         << endl << " [--minObjBkgRatio ratio(double)]"
         << endl << " [--groundtruthImgExtension string]"
         << endl << " [--groundtruthTxtExtension string]"
         << endl << " [--classificationMethod EXTRATREE|EXTRATREE_SOFT_THRESHOLD|EXTRATREE_2ATT|SVM]"
         << endl << " [--nbTrees number(int)]"
         << endl << " [--useSimpleScore]"
         << endl << " [--scoreThreshold number(double)]"
         << endl << " [--nbScoreTries number(int)]"
         << endl << " [--pruningThreshold number(double)]"
         << endl << " [--maxPruningPercentage number(int)]"
         << endl << " [--maxTerminalNodes number(int)]"
         << endl << " [--testingFeaturesPerImage number(int)]"
         << endl << " [--pdfIncreaseMethod NONE|CUBE|SQUARE|CIRCLE [incAmount [decAmount [param1 [param2]]]]"
         << endl << " [--pdfInitValue number(int)]"
         << endl << " [--pdfMaxValue number(int)]"
         << endl << " [--usePdfRadiusAdaption]"
         << endl << " [--use2ndStageClassifier NONE|BINARY|PUREBINARY|COUNT|PURECOUNT|PUREBINARYENTROPY|PURENORMCOUNT]"
         << endl << " [--doTreeWeightUpdate]"
         << endl << " [-o outputfile]"
         << endl << " [--loadClassifier filename]"
         << endl << " [--saveClassifier filename]"
         << endl << " [--loadFeatureVectors directory]"
         << endl << " [--saveFeatureVectors directory]"
         << endl << " [--createDebugImages number [directory]]"
         << endl << " [--createPDFImages number [directory]]"
         << endl << " [--visualizeTree file]"
         << endl << " (--trainDirectory [imNbStart imCountPerClass] directory | --trainFilelist filelist labellist | -t (trainingImage classLabel)*)"
         << endl << " (--classifyDirectory [imNbStart imCountPerClass] directory | --classifyFilelist filelist labellist -c (classificationImage)*)" << endl;
      return "NICEEXIT";
    }
    
    if (actParam.compare("--configFile")==0) {
      if (i+1 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--configFile\" - not enough arguments");
      } else {
        actParam = args[++i];
        if (fileExists(actParam)) {
          ifstream inFile (actParam.c_str());
          if (!inFile) {
            paramError = true;
            errorMsg.append("\nError in parameter \"--configFile\" - error while reading file");
          } else {
            string s;
            vector<std::string> params;
            while (!inFile.eof())
            {
              getline (inFile,s);
              if (s.length() > 1) {
                try {
                  explode(s, " ", &params);
                } catch (int e) {
                  continue;
                }
              } //length > 1
            } //reading lines
            string err = evaluateParams(params, mainParams, extraTreeParams, svmParams, pdfParams);
            if (err != "") {
              paramError = true;
              errorMsg = err;
            }
          } //successfully opened file
        } else {
          paramError = true;
          errorMsg.append("\nError in parameter \"--configFile\" - specified file not found");
        }
      }
      continue;
    }
#ifdef HAVE_SIFT
    if (actParam.compare("--SiftConfigFile")==0) {
      if (i+1 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--SiftConfigFile\" - not enough arguments");
      } else {
        actParam = args[++i];
        if (fileExists(actParam)) {
          mainParams->siftCfgFile = actParam;
        } else {
          paramError = true;
          errorMsg.append("\nError in parameter \"--SiftConfigFile\" - specified file not found");
        }
      }
      continue;
    }
#endif
    if (actParam.compare("--loadClassifier")==0) {
      if (i+1 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--loadClassifier\" - not enough arguments");
      } else {
        actParam = args[++i];
        if (fileExists(actParam)) {
          mainParams->loadClassifierFile = actParam;
        } else {
          paramError = true;
          errorMsg.append("\nError in parameter \"--loadClassifier\" - specified file not found");
        }
      }
      continue;
    }
    if (actParam.compare("--saveClassifier")==0) {
      if (i+1 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--saveClassifier\" - not enough arguments");
      } else {
        actParam = args[++i];
        mainParams->saveClassifierFile = actParam;
      }
      continue;
    }
    if (actParam.compare("--loadFeatureVectors")==0) {
      if (i+1 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--loadFeatureVectors\" - not enough arguments");
      } else {
        actParam = args[++i];
        mainParams->readFeatureVectors = true;
        mainParams->featureVecInputDir = actParam;
        // set training files
        string fname = actParam + "trainingfilelist.txt";
        string iname = actParam + "trainingfilelabels.txt";
        if (fileExists(fname) && fileExists(iname)) {
          ifstream filelist(fname.c_str());
          ifstream infolist(iname.c_str());
          mainParams->trainingFiles.clear();
          mainParams->trainingFileLabels.reset();
          while (!filelist.eof() && !infolist.eof()) {
            string fline;
            string iline;
            getline(filelist,fline);
            getline(infolist,iline);
            if (fileExists(fline)) {
              int pos = fline.rfind(".desc");
              fline.erase(pos,5);
              TrainingFile currFile;
              currFile.fileName = fline;
              from_string<ClassLabel>(currFile.classLabel, iline, std::dec);
              currFile.groundtruthImgFile = "";
              currFile.groundtruthTxtFile = "";
              mainParams->trainingFiles.push_back(currFile);
              mainParams->trainingFileLabels.increaseCountOfLabel(currFile.classLabel);
            }
          }
          filelist.close();
        }
        // set testing files
        fname = actParam + "testingfilelist.txt";
        iname = actParam + "testingfilelabels.txt";
        if (fileExists(fname) && fileExists(iname)) {
          ifstream filelist(fname.c_str());
          ifstream infolist(iname.c_str());
          mainParams->classificationFiles.clear();
          mainParams->classificationFileLabels.reset();
          while (!filelist.eof() && !infolist.eof()) {
            string fline;
            string iline;
            getline(filelist,fline);
            getline(infolist,iline);
            if (fileExists(fline)) {
              int pos = fline.rfind(".desc");
              fline.erase(pos,5);
              ClassificationFile currFile;
              currFile.fileName = fline;
              from_string<ClassLabel>(currFile.classLabel, iline, std::dec);
              mainParams->classificationFiles.push_back(currFile);
              mainParams->classificationFileLabels.increaseCountOfLabel(currFile.classLabel);
            }
          }
          filelist.close();
        }
      }
      continue;
    }
    if (actParam.compare("--saveFeatureVectors")==0) {
      if (i+1 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--saveFeatureVectors\" - not enough arguments");
      } else {
        actParam = args[++i];
        mainParams->featureVecOutputDir = actParam;
        mainParams->saveFeatureVectors = true;
      }
      continue;
    }
    if (actParam.compare("--trainFilelist")==0) {
      if (i+2 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--trainFilelist\" - not enough arguments");
      } else {
        actParam = args[++i];
        string fname = actParam;
        //cout << endl << "trainfiles:" << fname << flush;
        actParam = args[++i];
        string iname = actParam;
        //cout << endl << "trainlabels:" << iname << flush;
        if (fileExists(fname) && fileExists(iname)) {
          ifstream filelist(fname.c_str());
          ifstream infolist(iname.c_str());
          mainParams->trainingFiles.clear();
          mainParams->trainingFileLabels.reset();
          while (!filelist.eof() && !infolist.eof()) {
            string fline;
            string iline;
            getline(filelist,fline);
            getline(infolist,iline);
            if (fileExists(fline)) {
              TrainingFile currFile;
              currFile.fileName = fline;
              from_string<ClassLabel>(currFile.classLabel, iline, std::dec);
              // ATTENTION: special semantics for ClassLabel 0/1: 0="no Class", 1="background"
#ifdef TESTHACK
  currFile.classLabel+=2;
#endif
              currFile.groundtruthImgFile = "";
              currFile.groundtruthTxtFile = "";
              mainParams->trainingFiles.push_back(currFile);
              mainParams->trainingFileLabels.increaseCountOfLabel(currFile.classLabel);
            }
          }
          filelist.close();
        }
      }
      continue;
    }
    if (actParam.compare("--classifyFilelist")==0) {
      if (i+2 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--classifyFilelist\" - not enough arguments");
      } else {
        actParam = args[++i];
        string fname = actParam;
        //cout << endl << "trainfiles:" << fname << flush;
        actParam = args[++i];
        string iname = actParam;
        //cout << endl << "trainlabels:" << iname << flush;
        if (fileExists(fname) && fileExists(iname)) {
          ifstream filelist(fname.c_str());
          ifstream infolist(iname.c_str());
          mainParams->classificationFiles.clear();
          mainParams->classificationFileLabels.reset();
          while (!filelist.eof() && !infolist.eof()) {
            string fline;
            string iline;
            getline(filelist,fline);
            getline(infolist,iline);
            if (fileExists(fline)) {
              ClassificationFile currFile;
              currFile.fileName = fline;
              from_string<ClassLabel>(currFile.classLabel, iline, std::dec);
              // ATTENTION: special semantics for ClassLabel 0/1: 0="no Class", 1="background"
#ifdef TESTHACK
  currFile.classLabel+=2;
#endif
              mainParams->classificationFiles.push_back(currFile);
              mainParams->classificationFileLabels.increaseCountOfLabel(currFile.classLabel);
            }
          }
          filelist.close();
        }
      }
      continue;
    }
    if (actParam.compare("--visualizeTree")==0) {
      if (i+1 >= argcnt) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--visualizeTree\" - not enough arguments");
      } else {
        actParam = args[++i];
        mainParams->treeLeafesOutputFile = actParam;
      }
      continue;
    }
    if (actParam.compare("--classificationMethod")==0) {
      actParam = args[++i];
      mainParams->classificationMethod = asClassificationMethod(actParam);
      if ((mainParams->classificationMethod == CM_EXTRA_TREE_SINGLE_ATT) && (actParam != "EXTRATREE")) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--classificationMethod\"");
      }
      if (mainParams->classificationMethod == CM_EXTRA_TREE_COMPARE_ATT)
        extraTreeParams->useCompareAtt = true;
      if (mainParams->classificationMethod == CM_EXTRA_TREE_SOFT_THRESHOLD)
        extraTreeParams->useSoftThreshold = true;
      continue;
    }
    if (actParam.compare("--totalTrainingFeatures")==0) {
      actParam = args[++i];
      if (!from_string<unsigned int>(mainParams->nbOfTrainingWindowsTotal, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--totalTrainingFeatures\"");
      }
      continue;
    }
    if (actParam.compare("--testingFeaturesPerImage")==0) {
      actParam = args[++i];
      if (!from_string<unsigned int>(mainParams->nbOfTestingWindowsPerImg, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--testingFeaturesPerImage\"");
      }
      continue;
    }
    if (actParam.compare("--extractionMethod")==0) {
      actParam = args[++i];
      mainParams->extMethod = FeatureExtractor::asExtractionMethod(actParam);
      if ((mainParams->extMethod == FeatureExtractor::ColorHSL) && (!(actParam == "HSL"))) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--extractionMethod\"");
      }
      continue;
    }
    if (actParam.compare("--maxSubwindowSize")==0) {
      actParam = args[++i];
      if (!from_string<int>(mainParams->maxSubwindowSize, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--maxSubwindowSize\"");
      }
      continue;
    }
    if (actParam.compare("--nbTrees")==0) {
      actParam = args[++i];
      if (!from_string<int>(extraTreeParams->nbOfTrees, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--nbOfTrees\"");
      }
      continue;
    }
    if (actParam.compare("-t")==0) {
      cout << endl << "searching training files..." << flush;
      parseState = 30;
      continue;
    }
    if (actParam.compare("--trainDirectory")==0) {
      cout << endl << "searching training files..." << flush;
      parseState = 35;
      currInt1 = -1; currInt2 = -1;
      continue;
    }
    if (actParam.compare("-c")==0) {
      cout << endl << "searching classification files..." << flush;
      parseState = 40;
      continue;
    }
    if (actParam.compare("--classifyDirectory")==0) {
      cout << endl << "searching classification files..." << flush;
      currInt1 = -1; currInt2 = -1;
      parseState = 45;
      continue;
    }
    if (actParam.compare("-o")==0) {
      actParam = args[++i];
      mainParams->outputFile = actParam;
      continue;
    }
    if (actParam.compare("--scoreThreshold")==0) {
      actParam = args[++i];
      if (!from_string<double>(extraTreeParams->scoreThreshold, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--scoreThreshold\"");
      }
      continue;
    }
    if (actParam.compare("--nbScoreTries")==0) {
      actParam = args[++i];
      if (!from_string<int>(extraTreeParams->nbScoreTries, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--nbScoreTries\"");
      }
      continue;
    }
    if (actParam.compare("--pruningThreshold")==0) {
      actParam = args[++i];
      if (!from_string<double>(extraTreeParams->pruningThreshold, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--pruningThreshold\"");
      }
      continue;
    }
    if (actParam.compare("--minObjBkgRatio")==0) {
      actParam = args[++i];
      if (!from_string<double>(mainParams->minObjBgrRatio, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--minObjBgrRatio\"");
      }
      continue;
    }
    if (actParam.compare("--groundtruthImgExtension")==0) {
      actParam = args[++i];
      mainParams->groundtruthImgExtension = actParam;
      continue;
    }
    if (actParam.compare("--groundtruthTxtExtension")==0) {
      actParam = args[++i];
      mainParams->groundtruthTxtExtension = actParam;
      continue;
    }
    if (actParam.compare("--maxPruningPercentage")==0) {
      actParam = args[++i];
      if (!from_string<unsigned int>(extraTreeParams->maxNbPrunedNodes, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--maxPruningPercentage\"");
      }
      continue;
    }
    if (actParam.compare("--maxTerminalNodes")==0) {
      actParam = args[++i];
      if (!from_string<unsigned int>(extraTreeParams->maxNbTerminalNodes, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--maxTerminalNodes\"");
      }
      continue;
    }
    if (actParam.compare("--useSimpleScore")==0) {
      extraTreeParams->useSimpleScore = true;
      continue;
    }
    if (actParam.compare("--equalizeTrainingFeaturesPerClass")==0) {
      mainParams->useSameNbTrWinPerClass = true;
      continue;
    }
    if (actParam.compare("--pdfIncreaseMethod")==0) {
      parseState = 70;
      continue;
    }
    if (actParam.compare("--pdfInitValue")==0) {
      actParam = args[++i];
      if (!from_string<unsigned int>(pdfParams->initValue, actParam, std::dec)) {
                paramError = true;
                errorMsg.append("\nError in parameter \"--pdfInitValue\"");
              }
      continue;
    }
    if (actParam.compare("--pdfMaxValue")==0) {
      actParam = args[++i];
      if (!from_string<unsigned int>(pdfParams->maxValue, actParam, std::dec)) {
                paramError = true;
                errorMsg.append("\nError in parameter \"--pdfMaxValue\"");
              }
      continue;
    }
    if (actParam.compare("--usePdfRadiusAdaption")==0) {
      mainParams->usePDFRadiusAdaption = true;
      continue;
    }
    if (actParam.compare("--use2ndStageClassifier")==0) {
      actParam = args[++i];
      mainParams->use2ndStageClassifier = asClassifierStage2(actParam);
      if ((mainParams->use2ndStageClassifier == CL2_NONE) && (actParam.compare("NONE")!=0)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--use2ndStageClassifier\"");
      }
      continue;
    }
    if (actParam.compare("--doTreeWeightUpdate")==0) {
      mainParams->doTreeWeightUpdate = true;
      continue;
    }
    if (actParam.compare("--createDebugImages")==0) {
      actParam = args[++i];
      if (!from_string<int>(mainParams->nbDebugImages, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--createDebugImages\"");
      } else {
        actParam = args[++i];
        mainParams->debugImageDirectory = actParam;
      }
      continue;
    }
    if (actParam.compare("--createPDFImages")==0) {
      actParam = args[++i];
      if (!from_string<int>(mainParams->nbPDFImages, actParam, std::dec)) {
        paramError = true;
        errorMsg.append("\nError in parameter \"--createPDFImages\"");
      } else {
        actParam = args[++i];
        mainParams->PDFImageDirectory = actParam;
      }
      continue;
    }
    
    if (parseState == 0) {
      paramError = true;
      errorMsg.append("\nError in parameter \"" + actParam + "\"");
    }
    
    //cout << "entering switch"<<endl;
    switch (parseState) {
      case 30:currFile = actParam;
              parseState = 31;
              break;
      case 31:{
                ClassLabel fileLabel;
                if (!from_string<ClassLabel>(fileLabel, actParam, std::dec)) {
                  paramError = true;
                  errorMsg.append("\nError in parameter \"-t\"");
                }
                else
                {
                  list<string> fileList;
                  getFiles(&fileList, currFile, &(mainParams->ImgFileExtensions));
                  //cout << fileList.size() << " files found  " << flush;
                  for (list<string>::iterator p=fileList.begin(); p!=fileList.end(); ++p)
                  {
                    TrainingFile currFile;
                    currFile.fileName = *p;
                    currFile.classLabel = fileLabel;
                    currFile.groundtruthImgFile = "";
                    currFile.groundtruthTxtFile = "";
                    mainParams->trainingFiles.push_back(currFile);
                    mainParams->trainingFileLabels.increaseCountOfLabel(currFile.classLabel);
                  }
                }
                parseState = 31;
              }
              break;
      case 35:{
                // check weather string is an integer
                int tmpint;
                if (from_string<int>(tmpint, actParam, std::dec)) {
                  if (currInt1 == -1)
                    currInt1 = tmpint;
                  else
                    currInt2 = tmpint;
                  break; // break the current switch and continue with next argument
                }
                string currDir = actParam;
                list<string> dirList;
                // get all subdirectories that contain the training images of the different classes
                if (getSubdirs(&dirList, currDir) > 0)
                {
                  paramError = true;
                  errorMsg.append("\nError in parameter \"--trainDirectory\"");
                  break;
                }
                for (list<string>::iterator p=dirList.begin(); p!=dirList.end(); ++p)
                {
                  string currSubDir = *p;
                  // remove "/"
                  unsigned int spos = currSubDir.find("/",0);
                  if (spos != string::npos)
                    currSubDir.erase(spos);
                  // extract class label "/"
                  ClassLabel dirLabel;
                  if (!from_string<ClassLabel>(dirLabel, currSubDir, std::dec))
                  {
                    paramError = true;
                    errorMsg.append("\nError in parameter \"--trainDirectory\"");
                    break;
                  }
                  // get all the files in this directory and add to list (not in subdirectories)
                  list<string> fileList;
                  getFiles(&fileList, currDir+currSubDir+"/", false, &(mainParams->ImgFileExtensions));
                  //cout << endl << fileList.size() << " files found in " << currDir+currSubDir+"/" << flush;
                  int currPos = 0;
                  int addedFiles = 0;
                  for (list<string>::iterator q=fileList.begin(); q!=fileList.end(); ++q)
                  {
                    if ((currPos >= currInt1) && ((addedFiles < currInt2) || (currInt2 == -1))) {
                      TrainingFile currFile;
                      //cout << "file " << *q << " added for class " << dirLabel << endl;
                      currFile.fileName = *q;
                      string maskfilename = extractPath(*q)+mainParams->masksPath+extractFileName(*q)+mainParams->groundtruthImgExtension;
                      if (fileExists(maskfilename)) {
                        currFile.groundtruthImgFile = maskfilename;
                        //currFile.groundtruthImgFile = "";
                      }
                      else
                        currFile.groundtruthImgFile = "";
                      maskfilename = extractPath(*q)+extractFileName(*q,false)+mainParams->groundtruthTxtExtension;
                      if (fileExists(maskfilename)) {
                        currFile.groundtruthTxtFile = maskfilename;
                      }
                      else
                        currFile.groundtruthTxtFile = "";
                      currFile.classLabel = dirLabel;
                      mainParams->trainingFiles.push_back(currFile);
                      mainParams->trainingFileLabels.increaseCountOfLabel(currFile.classLabel);
                      addedFiles++;
                    }
                    currPos++;
                  }
                }
              } //-end for dirList
              parseState = 0;
              break;
      case 40:{
                list<string> fileList;
                getFiles(&fileList, actParam, &(mainParams->ImgFileExtensions));
                for (list<string>::iterator p=fileList.begin(); p!=fileList.end(); ++p)
                {
                  ClassificationFile currFile;
                  currFile.fileName = *p;
                  currFile.classLabel = NULL_CLASS_LABEL;
                  mainParams->classificationFiles.push_back(currFile);
                  mainParams->classificationFileLabels.increaseCountOfLabel(currFile.classLabel);
                }
              }
              parseState = 40;
              break;
      case 45: {
                // check weather string is an integer
                int tmpint;
                if (from_string<int>(tmpint, actParam, std::dec)) {
                  if (currInt1 == -1)
                    currInt1 = tmpint;
                  else
                    currInt2 = tmpint;
                  break; // break the current switch and continue with next argument
                }
                string currDir = actParam;
                list<string> dirList;
                // get all subdirectories that contain the classifying images of the different classes
                if (getSubdirs(&dirList, currDir) > 0)
                {
                  paramError = true;
                  errorMsg.append("\nError in parameter \"--classifyDirectory\"");
                  break;
                }
                for (list<string>::iterator p=dirList.begin(); p!=dirList.end(); ++p)
                {
                  string currSubDir = *p;
                  // remove "/"
                  unsigned int spos = currSubDir.find("/",0);
                  if (spos != string::npos)
                    currSubDir.erase(spos);
                  // extract class label "/"
                  ClassLabel dirLabel;
                  if (!from_string<ClassLabel>(dirLabel, currSubDir, std::dec))
                  {
                    paramError = true;
                    errorMsg.append("\nError in parameter \"--classifyDirectory\"");
                    break;
                  }
                  // get all the files in this directory and add to list (not in subdirectories)
                  list<string> fileList;
                  getFiles(&fileList, currDir+currSubDir+"/", false, &(mainParams->ImgFileExtensions));
                  int currPos = 0;
                  int addedFiles = 0;
                  for (list<string>::iterator q=fileList.begin(); q!=fileList.end(); ++q)
                  {
                    if ((currPos >= currInt1) && ((addedFiles < currInt2) || (currInt2 == -1))) {
                      ClassificationFile currFile;
                      currFile.fileName = *q;
                      currFile.classLabel = dirLabel;
                      mainParams->classificationFiles.push_back(currFile);
                      mainParams->classificationFileLabels.increaseCountOfLabel(currFile.classLabel);
                      addedFiles++;
                    }
                    currPos++;
                  }
                }
              }
              break;
      case 70:{
              pdfParams->incMethod = PDF::asIncMethod(actParam);
              if ((pdfParams->incMethod == PDF::imNone) && (actParam != "NONE")) {
                paramError = true;
                errorMsg.append("\nError in parameter \"--pdfIncreaseMethod\"");
              }
              else
                parseState = 71;
              }
              break;
      case 71:{
              if (!from_string<int>(mainParams->pdfIncAmount, actParam, std::dec)) {
                paramError = true;
                errorMsg.append("\nError in parameter \"--pdfIncreaseMethod\"");
              }
              else
                parseState = 72;
              }
              break;
      case 72:{
              if (!from_string<int>(mainParams->pdfDecAmount, actParam, std::dec)) {
                paramError = true;
                errorMsg.append("\nError in parameter \"--pdfIncreaseMethod\"");
              }
              else
                parseState = 73;
              }
              break;
      case 73:{
              if (!from_string<double>(mainParams->pdfIncParam1, actParam, std::dec)) {
                paramError = true;
                errorMsg.append("\nError in parameter \"--pdfIncreaseMethod\"");
              }
              else
                parseState = 74;
              }
              break;
      case 74:if (!from_string<double>(mainParams->pdfIncParam2, actParam, std::dec)) {
                paramError = true;
                errorMsg.append("\nError in parameter \"--pdfIncreaseMethod\"");
              }
              parseState = 0;
              break;
    } //end switch
  } //end for-loop

  return(errorMsg);
}

